@@ -23,8 +23,8 @@
 #include <ags/libags.h>
 #include <ags/libags-audio.h>
 
-#include <ags/X/ags_xorg_application_context.h>
 #include <ags/X/ags_ui_provider.h>
+#include <ags/X/ags_window.h>
 
 #include <ags/X/file/ags_simple_file.h>
 
@@ -39,10 +39,6 @@
 #include <libinstpatch/libinstpatch.h>
 #endif
 
-#ifdef AGS_WITH_QUARTZ
-#include <gtkmacintegration/gtkosxapplication.h>
-#endif
-
 #include <libxml/parser.h>
 #include <libxml/xlink.h>
 #include <libxml/xpath.h>
@@ -63,10 +59,7 @@
 
 #include <gdk/gdk.h>
 
-#ifndef AGS_WITH_QUARTZ
 #include <fontconfig/fontconfig.h>
-#endif
-
 #include <math.h>
 
 void ags_gui_thread_signal_handler(int signr);
@@ -173,7 +166,7 @@ ags_gui_thread_get_type()
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_gui_thread;
+    GType ags_type_gui_thread = 0;
 
     static const GTypeInfo ags_gui_thread_info = {
       sizeof (AgsGuiThreadClass),
@@ -202,7 +195,7 @@ ags_gui_thread_get_type()
 				AGS_TYPE_CONNECTABLE,
 				&ags_connectable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_gui_thread);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_gui_thread);
   }
 
   return g_define_type_id__volatile;
@@ -501,7 +494,6 @@ ags_gui_thread_run(AgsThread *thread)
 {
   AgsGuiThread *gui_thread;
 
-  AgsMutexManager *mutex_manager;
   AgsThread *main_loop;
   AgsPollingThread *polling_thread;
   AgsPollFd *poll_fd;
@@ -517,14 +509,10 @@ ags_gui_thread_run(AgsThread *thread)
 
   gboolean some_ready;
 
-  pthread_mutex_t *application_mutex;
   pthread_mutex_t *mutex;
 
   /*  */
   gui_thread = AGS_GUI_THREAD(thread);
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
     
   /* real-time setup */  
   main_loop = ags_thread_get_toplevel(thread);
@@ -881,28 +869,19 @@ ags_gui_thread_launch(AgsGuiThread *gui_thread,
 {
   AgsApplicationContext *application_context;
   
-  AgsMutexManager *mutex_manager;
   AgsThread *audio_loop, *task_thread;
 
   AgsConfig *config;
 
   GList *start_queue;  
 
-  pthread_mutex_t *application_mutex;
-
   application_context = ags_application_context_get_instance();
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
     
   /* get threads, thread pool and config */
-  pthread_mutex_lock(application_mutex);
-  
-  audio_loop = (AgsThread *) application_context->main_loop;
-
-  config = application_context->config;
-  
-  pthread_mutex_unlock(application_mutex);
+  g_object_get(application_context,
+	       "main-loop", &audio_loop,
+	       "config", &config,
+	       NULL);
 
   task_thread = ags_thread_find_type(audio_loop,
 				     AGS_TYPE_TASK_THREAD);
@@ -934,29 +913,20 @@ ags_gui_thread_launch_filename(AgsGuiThread *gui_thread,
 {
   AgsApplicationContext *application_context;
 
-  AgsMutexManager *mutex_manager;
   AgsThread *audio_loop, *task_thread;
 
   AgsConfig *config;
 
   GList *start_queue;  
 
-  pthread_mutex_t *application_mutex;
-
   application_context = ags_application_context_get_instance();
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
     
   /* get threads, thread pool and config */
-  pthread_mutex_lock(application_mutex);
-
-  audio_loop = (AgsThread *) application_context->main_loop;
-  task_thread = (AgsThread *) application_context->task_thread;
-
-  config = application_context->config;
-  
-  pthread_mutex_unlock(application_mutex);
+  g_object_get(application_context,
+	       "main-loop", &audio_loop,
+	       "task-thread", &task_thread,
+	       "config", &config,
+	       NULL);
 
   /* open file */
   if(g_strcmp0(ags_config_get_value(config,
@@ -1046,31 +1016,20 @@ ags_gui_thread_timer_launch(AgsGuiThread *gui_thread,
 {
   AgsApplicationContext *application_context;
   
-  AgsMutexManager *mutex_manager;
   AgsThread *audio_loop, *task_thread;
 
   AgsConfig *config;
 
   GList *start_queue;  
 
-  pthread_mutex_t *application_mutex;
-
   application_context = ags_application_context_get_instance();
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
   /* get threads, thread pool and config */
-  pthread_mutex_lock(application_mutex);
-  
-  audio_loop = (AgsThread *) application_context->main_loop;
-
-  config = application_context->config;
-  
-  pthread_mutex_unlock(application_mutex);
-
-  task_thread = ags_thread_find_type(audio_loop,
-				     AGS_TYPE_TASK_THREAD);
+  g_object_get(application_context,
+	       "main-loop", &audio_loop,
+	       "task-thread", &task_thread,
+	       "config", &config,
+	       NULL);
   
   /* autosave thread */
   if(!g_strcmp0(ags_config_get_value(config,
@@ -1097,30 +1056,20 @@ ags_gui_thread_timer_launch_filename(AgsGuiThread *gui_thread,
 {
   AgsApplicationContext *application_context;
   
-  AgsMutexManager *mutex_manager;
-
   AgsThread *audio_loop, *task_thread;
 
   AgsConfig *config;
       
   GList *start_queue;
 
-  pthread_mutex_t *application_mutex;
-
   application_context = ags_application_context_get_instance();
   
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
   /* get thread, thread pool and config */
-  pthread_mutex_lock(application_mutex);
-
-  audio_loop = (AgsThread *) application_context->main_loop;
-  task_thread = (AgsThread *) application_context->task_thread;
-
-  config = application_context->config;
-  
-  pthread_mutex_unlock(application_mutex);
+  g_object_get(application_context,
+	       "main-loop", &audio_loop,
+	       "task-thread", &task_thread,
+	       "config", &config,
+	       NULL);
 
   /* open file */
   if(g_strcmp0(ags_config_get_value(config,
@@ -1201,26 +1150,17 @@ ags_gui_thread_animation_prepare(GSource *source,
 
   AgsGuiThread *gui_thread;
   
-  AgsMutexManager *mutex_manager;
+  AgsThread *main_loop;
 
   AgsLog *log;
 
   guint nth;
-
-  static gboolean initial_run = TRUE;
-  
-  pthread_mutex_t *application_mutex;
   
   application_context = ags_application_context_get_instance();
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  pthread_mutex_lock(application_mutex);
-
-  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
-
-  pthread_mutex_unlock(application_mutex);
+  main_loop = AGS_APPLICATION_CONTEXT(application_context)->main_loop;
+  
+  gui_thread = ags_thread_find_type(main_loop,
+				    AGS_TYPE_GUI_THREAD);
   
   log = ags_log_get_instance();
   
@@ -1230,15 +1170,12 @@ ags_gui_thread_animation_prepare(GSource *source,
 
   pthread_mutex_unlock(log->mutex);
 
-  if(initial_run ||
-     (gui_thread != NULL && nth > gui_thread->nth_message) ||
+  if(nth > gui_thread->nth_message ||
      !ags_ui_provider_get_show_animation(AGS_UI_PROVIDER(application_context))){
     if(timeout_ != NULL){
       *timeout_ = 0;
     }
 
-    initial_run = FALSE;
-
     return(TRUE);
   }else{
     if(timeout_ != NULL){
@@ -1256,27 +1193,18 @@ ags_gui_thread_animation_check(GSource *source)
 
   AgsGuiThread *gui_thread;
   
-  AgsMutexManager *mutex_manager;
+  AgsThread *main_loop;
 
   AgsLog *log;
 
   guint nth;
 
-  static gboolean initial_run = TRUE;
-
-  pthread_mutex_t *application_mutex;
-
   application_context = ags_application_context_get_instance();
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  pthread_mutex_lock(application_mutex);
-
-  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
-
-  pthread_mutex_unlock(application_mutex);
+  main_loop = AGS_APPLICATION_CONTEXT(application_context)->main_loop;
   
+  gui_thread = ags_thread_find_type(main_loop,
+				    AGS_TYPE_GUI_THREAD);
+
   log = ags_log_get_instance();
   
   pthread_mutex_lock(log->mutex);
@@ -1285,11 +1213,8 @@ ags_gui_thread_animation_check(GSource *source)
 
   pthread_mutex_unlock(log->mutex);
 
-  if(initial_run ||
-     (gui_thread != NULL && nth > gui_thread->nth_message) ||
+  if(nth > gui_thread->nth_message ||
      !ags_ui_provider_get_show_animation(AGS_UI_PROVIDER(application_context))){
-    initial_run = FALSE;
-
     return(TRUE);
   }else{
     return(FALSE);
@@ -1310,41 +1235,18 @@ ags_gui_thread_animation_dispatch(GSource *source,
 
   GMainContext *main_context;
   
+  AgsThread *main_loop;
   AgsTaskThread *task_thread;
-  AgsMutexManager *mutex_manager;
-
-  gboolean retval;
-  
-  pthread_mutex_t *application_mutex;
 
   application_context = ags_application_context_get_instance();
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  pthread_mutex_lock(application_mutex);
-
-  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
-
-  pthread_mutex_unlock(application_mutex);
+  main_loop = AGS_APPLICATION_CONTEXT(application_context)->main_loop;
+  
+  gui_thread = ags_thread_find_type(main_loop,
+				    AGS_TYPE_GUI_THREAD);
 
   main_context = g_main_context_default();
 
-  if(!g_main_context_acquire(main_context)){
-    gboolean got_ownership = FALSE;
-
-    g_mutex_lock(&(gui_thread->mutex));
-    
-    while(!got_ownership){
-      got_ownership = g_main_context_wait(main_context,
-					  &(gui_thread->cond),
-					  &(gui_thread->mutex));
-    }
-
-    g_mutex_unlock(&(gui_thread->mutex));
-  }
-  
-  if(window == NULL){            
+  if(window == NULL){
     window = g_object_new(GTK_TYPE_WINDOW,
 			  "app-paintable", TRUE,
 			  "type", GTK_WINDOW_TOPLEVEL,
@@ -1365,24 +1267,21 @@ ags_gui_thread_animation_dispatch(GSource *source,
 		     G_CALLBACK(ags_gui_thread_do_animation_callback), gui_thread);
   }
 
-  if(ags_ui_provider_get_show_animation(AGS_UI_PROVIDER(application_context))){
-    gtk_widget_queue_draw(widget);
+  gtk_widget_queue_draw(widget);
 
-    g_main_context_iteration(main_context,
-			     FALSE);
+  g_main_context_iteration(main_context,
+  			   FALSE);
   
-    retval = G_SOURCE_CONTINUE;
-  }else{    
-    /*  */
-    gtk_widget_hide(window);
+  if(ags_ui_provider_get_show_animation(AGS_UI_PROVIDER(application_context))){
+    return(G_SOURCE_CONTINUE);
+  }else{
+    gtk_widget_destroy(window);
     window = NULL;
 
-    retval = G_SOURCE_REMOVE;
+    gtk_widget_show_all(ags_ui_provider_get_window(AGS_UI_PROVIDER(application_context)));
+
+    return(G_SOURCE_REMOVE);
   }
-
-  g_main_context_release(main_context);
-
-  return(retval);
 }
 
 gboolean
@@ -1390,28 +1289,22 @@ ags_gui_thread_sync_task_prepare(GSource *source,
 				 gint *timeout_)
 {  
   AgsGuiThread *gui_thread;
-  
-  AgsMutexManager *mutex_manager;
-  AgsTaskThread *task_thread;
 
   AgsApplicationContext *application_context;
-
-  pthread_mutex_t *application_mutex;
+  
+  AgsThread *main_loop;
+  AgsTaskThread *task_thread;
 
   application_context = ags_application_context_get_instance();
+  main_loop = application_context->main_loop;
   
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  task_thread = ags_thread_find_type(main_loop,
+				     AGS_TYPE_TASK_THREAD);
 
-  pthread_mutex_lock(application_mutex);
-
-  task_thread = ags_concurrency_provider_get_task_thread(AGS_CONCURRENCY_PROVIDER(application_context));
+  gui_thread = ags_thread_find_type(main_loop,
+				    AGS_TYPE_GUI_THREAD);
   
-  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
-
-  pthread_mutex_unlock(application_mutex);
-  
-  if(gui_thread != NULL && gui_thread->queued_sync > 0){
+  if(gui_thread->queued_sync > 0){
     if(timeout_ != NULL){
       *timeout_ = 0;
     }
@@ -1431,25 +1324,19 @@ ags_gui_thread_sync_task_check(GSource *source)
 {
   AgsGuiThread *gui_thread;
 
-  AgsMutexManager *mutex_manager;
-  AgsTaskThread *task_thread;
-
   AgsApplicationContext *application_context;
   
-  pthread_mutex_t *application_mutex;
-  
+  AgsThread *main_loop;
+  AgsTaskThread *task_thread;
+
   application_context = ags_application_context_get_instance();
+  main_loop = application_context->main_loop;
   
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  task_thread = ags_thread_find_type(main_loop,
+				     AGS_TYPE_TASK_THREAD);
 
-  pthread_mutex_lock(application_mutex);
-
-  task_thread = ags_concurrency_provider_get_task_thread(AGS_CONCURRENCY_PROVIDER(application_context));
-  
-  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
-
-  pthread_mutex_unlock(application_mutex);
+  gui_thread = ags_thread_find_type(main_loop,
+				    AGS_TYPE_GUI_THREAD);
 
   if(gui_thread->queued_sync > 0){    
     return(TRUE);
@@ -1465,11 +1352,11 @@ ags_gui_thread_sync_task_dispatch(GSource *source,
 {
   AgsGuiThread *gui_thread;
 
-  AgsMutexManager *mutex_manager;
-  AgsTaskThread *task_thread;
-
   AgsApplicationContext *application_context;
 
+  AgsThread *main_loop;
+  AgsTaskThread *task_thread;
+
   GMainContext *main_context;
 
   gboolean do_sync;
@@ -1479,25 +1366,15 @@ ags_gui_thread_sync_task_dispatch(GSource *source,
     AGS_GUI_THREAD_SYNC_AVAILABLE_TIMEOUT,
   };
   
-  pthread_mutex_t *application_mutex;
-
   application_context = ags_application_context_get_instance();
+  main_loop = application_context->main_loop;
   
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  task_thread = ags_thread_find_type(main_loop,
+				     AGS_TYPE_TASK_THREAD);
 
-  pthread_mutex_lock(application_mutex);
+  gui_thread = ags_thread_find_type(main_loop,
+				    AGS_TYPE_GUI_THREAD);
 
-  task_thread = ags_concurrency_provider_get_task_thread(AGS_CONCURRENCY_PROVIDER(application_context));
-  
-  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
-
-  pthread_mutex_unlock(application_mutex);
-
-  if(gui_thread == NULL || task_thread == NULL){
-    return(G_SOURCE_CONTINUE);
-  }
-  
   main_context = gui_thread->main_context;
 
   if(ags_ui_provider_get_show_animation(AGS_UI_PROVIDER(application_context))){
@@ -1632,26 +1509,19 @@ ags_gui_thread_task_prepare(GSource *source,
 gboolean
 ags_gui_thread_task_check(GSource *source)
 {
-  AgsGuiThread *gui_thread;
-
-  AgsMutexManager *mutex_manager;
-  
   AgsApplicationContext *application_context;
 
-  pthread_mutex_t *application_mutex;
+  AgsGuiThread *gui_thread;
+  
+  AgsThread *main_loop;
 
   application_context = ags_application_context_get_instance();
+  main_loop = application_context->main_loop;
   
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  pthread_mutex_lock(application_mutex);
+  gui_thread = ags_thread_find_type(main_loop,
+				    AGS_TYPE_GUI_THREAD);
   
-  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
-
-  pthread_mutex_unlock(application_mutex);
-  
-  if(gui_thread != NULL && gui_thread->collected_task != NULL){
+  if(gui_thread->collected_task != NULL){
     return(TRUE);
   }else{
     return(FALSE);
@@ -1663,36 +1533,27 @@ ags_gui_thread_task_dispatch(GSource *source,
 			     GSourceFunc callback,
 			     gpointer user_data)
 {
-  AgsGuiThread *gui_thread;
-  
-  AgsMutexManager *mutex_manager;
-  AgsTaskThread *task_thread;
-
   AgsApplicationContext *application_context;
 
-  pthread_mutex_t *application_mutex;
+  AgsGuiThread *gui_thread;
+  
+  AgsThread *main_loop;
+  AgsTaskThread *task_thread;
 
   application_context = ags_application_context_get_instance();
+  main_loop = application_context->main_loop;
   
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  gui_thread = ags_thread_find_type(main_loop,
+				    AGS_TYPE_GUI_THREAD);
+  task_thread = ags_thread_find_type(main_loop,
+				     AGS_TYPE_TASK_THREAD);
 
-  pthread_mutex_lock(application_mutex);
+  ags_task_thread_append_tasks(task_thread,
+			       g_list_reverse(gui_thread->collected_task));
 
-  task_thread = ags_concurrency_provider_get_task_thread(AGS_CONCURRENCY_PROVIDER(application_context));
-  
-  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
+  gui_thread->collected_task = NULL;
 
-  pthread_mutex_unlock(application_mutex);
-  
-  if(gui_thread != NULL){
-    ags_task_thread_append_tasks(task_thread,
-				 g_list_reverse(gui_thread->collected_task));
-
-    gui_thread->collected_task = NULL;
-
-    ags_gui_thread_complete_task(gui_thread);
-  }
+  ags_gui_thread_complete_task(gui_thread);
   
   return(G_SOURCE_CONTINUE);
 }
@@ -1701,7 +1562,6 @@ gboolean
 ags_gui_thread_do_animation_callback(GtkWidget *widget, GdkEventExpose *event,
 				     AgsGuiThread *gui_thread)
 {
-  AgsApplicationContext *application_context;
   AgsLog *log;
 
   GdkRectangle rectangle;
@@ -1721,12 +1581,6 @@ ags_gui_thread_do_animation_callback(GtkWidget *widget, GdkEventExpose *event,
   guint i;
   guint nth = 0;
 
-  application_context = ags_application_context_get_instance();
-
-  if(!ags_ui_provider_get_show_animation(AGS_UI_PROVIDER(application_context))){
-    return(TRUE);
-  }
-  
   log = ags_log_get_instance();  
   
   if(filename == NULL){
@@ -2089,7 +1943,7 @@ ags_gui_thread_schedule_task_list(AgsGuiThread *gui_thread,
  *
  * Returns: the new #AgsGuiThread
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsGuiThread*
 ags_gui_thread_new()
